package com.midimind.core.hardware;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

public class DeviceAutodetector extends Thread {

	private List<DeviceAutodetectorListener> listeners = null;
	private long timeout = -1;
	private boolean interrupted = false;
	private Timer timeoutTimer = null;
	
	public DeviceAutodetector() {
		this.listeners = new ArrayList<DeviceAutodetectorListener>(); 
	}
	
	public DeviceAutodetector(long timeout) {
		this();
		this.timeout = timeout;
	}
	
	public void addDeviceAutodetectorListener(DeviceAutodetectorListener listener) {
		this.listeners.add(listener);
	}
	
	@Override
	public void run() {
		if (this.timeout > 0) {
			this.timeoutTimer = new Timer();
			this.timeoutTimer.schedule(new Timeout(this), this.timeout);
		}
		
		try {
			Set<Device> initialSet = new HashSet<Device>();
			
			for(Device device : Device.getAvailableDevices())
				initialSet.add(device);
	
			Device result = null;
			
			while((!interrupted) && (result == null))
				for(Device device : Device.getAvailableDevices())
					if (!initialSet.contains(device))
						result = device;
			
			if (interrupted)
				for(DeviceAutodetectorListener listener : this.listeners)
					listener.timeout();
			else
				for(DeviceAutodetectorListener listener : this.listeners)
					listener.deviceDetected(result);
		} catch (DeviceException e) {
			for(DeviceAutodetectorListener listener : this.listeners)
				listener.deviceDetectionFailed(e);
		} finally {
			this.timeoutTimer.cancel();
			this.interrupt();
		}
	}
	
	private void timeout() {
		this.interrupted = true;
	}
	
	private class Timeout extends TimerTask {

		private DeviceAutodetector target = null;
				
		public Timeout(DeviceAutodetector target) {
			super();
			this.target = target;
		}

		@Override
		public void run() {
			target.timeout();
			this.cancel();
		}
		
	}

}
