package com.midimind.core.hardware.ftdi;

import com.ftdichip.ftd2xx.DataBits;
import com.ftdichip.ftd2xx.Device;
import com.ftdichip.ftd2xx.DeviceDescriptor;
import com.ftdichip.ftd2xx.FTD2xxException;
import com.ftdichip.ftd2xx.FlowControl;
import com.ftdichip.ftd2xx.Parity;
import com.ftdichip.ftd2xx.Service;
import com.ftdichip.ftd2xx.StopBits;
import com.midimind.core.hardware.DeviceException;

public class FTDIDevice extends com.midimind.core.hardware.Device {

	private static final long READ_TIMEOUT = 1000;
	
	private Device device = null;
	private String id = null;
	private String description = null;
	
	public FTDIDevice() {
		super();
	}
	
	public FTDIDevice(Device device) throws FTD2xxException {
		super();
		
		DeviceDescriptor descriptor = device.getDeviceDescriptor();
		this.id = descriptor.getManufacturerID() + descriptor.getProductID();
		this.description = descriptor.getManufacturer() + " - " + 
			descriptor.getProductDescription() + " (" + id + ")";
		this.device = device;
	}

	@Override
	public String getDeviceID() {
		return id;
	}

	@Override
	public void close() throws DeviceException {
		try {
			this.device.close();
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}		
	}

	@Override
	public boolean isOpen() throws DeviceException {
		return this.device.isOpen();
	}

	@Override
	public void open() throws DeviceException {
		try {
			this.device.open();
			this.device.getPort().setFlowControl(FlowControl.NONE);
			this.device.getPort().setBaudRate(31250);
			this.device.getPort().setDataCharacteristics(
					DataBits.DATA_BITS_8, StopBits.STOP_BITS_1, Parity.NONE);
			this.device.setReadTimeout(READ_TIMEOUT);
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}		
	}

	@Override
	public void reset() throws DeviceException {
		try {
			this.device.reset();
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}
	}

	@Override
	public String toString() {
		return description;
	}

	@Override
	protected com.midimind.core.hardware.Device[] getAvailableDevices_internal()
			throws DeviceException {

		Device[] result = null;
		
		try {
			result = Service.listDevices();
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}
		
		FTDIDevice[] devices = new FTDIDevice[result.length];
		
		try {
			for(int i = 0; i < result.length; i++)
				devices[i] = new FTDIDevice(result[i]);
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}
		
		return devices;
	}

	@Override
	protected int read() throws DeviceException {
		try {
			return this.device.read();
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}
	}

	@Override
	protected void write(byte b) throws DeviceException {
		try {
			this.device.write(b);
		} catch (Exception e) {
			throw new DeviceException(e.getMessage(), FTDIError.getDeviceState(e));
		}		
	}

}
